// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/api/apikeys/v2/apikeys.proto

#include "google/cloud/apikeys/v2/internal/api_keys_metadata_decorator.h"
#include "google/cloud/grpc_options.h"
#include "google/cloud/internal/absl_str_cat_quiet.h"
#include "google/cloud/internal/api_client_header.h"
#include "google/cloud/internal/url_encode.h"
#include "google/cloud/status_or.h"
#include <google/api/apikeys/v2/apikeys.grpc.pb.h>
#include <memory>
#include <string>
#include <utility>
#include <vector>

namespace google {
namespace cloud {
namespace apikeys_v2_internal {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

ApiKeysMetadata::ApiKeysMetadata(
    std::shared_ptr<ApiKeysStub> child,
    std::multimap<std::string, std::string> fixed_metadata,
    std::string api_client_header)
    : child_(std::move(child)),
      fixed_metadata_(std::move(fixed_metadata)),
      api_client_header_(
          api_client_header.empty()
              ? google::cloud::internal::GeneratedLibClientHeader()
              : std::move(api_client_header)) {}

future<StatusOr<google::longrunning::Operation>>
ApiKeysMetadata::AsyncCreateKey(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::api::apikeys::v2::CreateKeyRequest const& request) {
  SetMetadata(*context, *options,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->AsyncCreateKey(cq, std::move(context), std::move(options),
                                request);
}

StatusOr<google::longrunning::Operation> ApiKeysMetadata::CreateKey(
    grpc::ClientContext& context, Options options,
    google::api::apikeys::v2::CreateKeyRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->CreateKey(context, options, request);
}

StatusOr<google::api::apikeys::v2::ListKeysResponse> ApiKeysMetadata::ListKeys(
    grpc::ClientContext& context, Options const& options,
    google::api::apikeys::v2::ListKeysRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->ListKeys(context, options, request);
}

StatusOr<google::api::apikeys::v2::Key> ApiKeysMetadata::GetKey(
    grpc::ClientContext& context, Options const& options,
    google::api::apikeys::v2::GetKeyRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->GetKey(context, options, request);
}

StatusOr<google::api::apikeys::v2::GetKeyStringResponse>
ApiKeysMetadata::GetKeyString(
    grpc::ClientContext& context, Options const& options,
    google::api::apikeys::v2::GetKeyStringRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->GetKeyString(context, options, request);
}

future<StatusOr<google::longrunning::Operation>>
ApiKeysMetadata::AsyncUpdateKey(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::api::apikeys::v2::UpdateKeyRequest const& request) {
  SetMetadata(
      *context, *options,
      absl::StrCat("key.name=", internal::UrlEncode(request.key().name())));
  return child_->AsyncUpdateKey(cq, std::move(context), std::move(options),
                                request);
}

StatusOr<google::longrunning::Operation> ApiKeysMetadata::UpdateKey(
    grpc::ClientContext& context, Options options,
    google::api::apikeys::v2::UpdateKeyRequest const& request) {
  SetMetadata(
      context, options,
      absl::StrCat("key.name=", internal::UrlEncode(request.key().name())));
  return child_->UpdateKey(context, options, request);
}

future<StatusOr<google::longrunning::Operation>>
ApiKeysMetadata::AsyncDeleteKey(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::api::apikeys::v2::DeleteKeyRequest const& request) {
  SetMetadata(*context, *options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncDeleteKey(cq, std::move(context), std::move(options),
                                request);
}

StatusOr<google::longrunning::Operation> ApiKeysMetadata::DeleteKey(
    grpc::ClientContext& context, Options options,
    google::api::apikeys::v2::DeleteKeyRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->DeleteKey(context, options, request);
}

future<StatusOr<google::longrunning::Operation>>
ApiKeysMetadata::AsyncUndeleteKey(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::api::apikeys::v2::UndeleteKeyRequest const& request) {
  SetMetadata(*context, *options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncUndeleteKey(cq, std::move(context), std::move(options),
                                  request);
}

StatusOr<google::longrunning::Operation> ApiKeysMetadata::UndeleteKey(
    grpc::ClientContext& context, Options options,
    google::api::apikeys::v2::UndeleteKeyRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->UndeleteKey(context, options, request);
}

StatusOr<google::api::apikeys::v2::LookupKeyResponse>
ApiKeysMetadata::LookupKey(
    grpc::ClientContext& context, Options const& options,
    google::api::apikeys::v2::LookupKeyRequest const& request) {
  SetMetadata(context, options);
  return child_->LookupKey(context, options, request);
}

StatusOr<google::longrunning::Operation> ApiKeysMetadata::GetOperation(
    grpc::ClientContext& context, Options const& options,
    google::longrunning::GetOperationRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->GetOperation(context, options, request);
}

future<StatusOr<google::longrunning::Operation>>
ApiKeysMetadata::AsyncGetOperation(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::longrunning::GetOperationRequest const& request) {
  SetMetadata(*context, *options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncGetOperation(cq, std::move(context), std::move(options),
                                   request);
}

future<Status> ApiKeysMetadata::AsyncCancelOperation(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::longrunning::CancelOperationRequest const& request) {
  SetMetadata(*context, *options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncCancelOperation(cq, std::move(context),
                                      std::move(options), request);
}

void ApiKeysMetadata::SetMetadata(grpc::ClientContext& context,
                                  Options const& options,
                                  std::string const& request_params) {
  context.AddMetadata("x-goog-request-params", request_params);
  SetMetadata(context, options);
}

void ApiKeysMetadata::SetMetadata(grpc::ClientContext& context,
                                  Options const& options) {
  google::cloud::internal::SetMetadata(context, options, fixed_metadata_,
                                       api_client_header_);
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace apikeys_v2_internal
}  // namespace cloud
}  // namespace google
