// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/redis/cluster/v1/cloud_redis_cluster.proto

#include "google/cloud/redis/cluster/v1/cloud_redis_cluster_client.h"
#include <memory>
#include <utility>

namespace google {
namespace cloud {
namespace redis_cluster_v1 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

CloudRedisClusterClient::CloudRedisClusterClient(
    std::shared_ptr<CloudRedisClusterConnection> connection, Options opts)
    : connection_(std::move(connection)),
      options_(
          internal::MergeOptions(std::move(opts), connection_->options())) {}
CloudRedisClusterClient::~CloudRedisClusterClient() = default;

StreamRange<google::cloud::redis::cluster::v1::Cluster>
CloudRedisClusterClient::ListClusters(std::string const& parent, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::ListClustersRequest request;
  request.set_parent(parent);
  return connection_->ListClusters(request);
}

StreamRange<google::cloud::redis::cluster::v1::Cluster>
CloudRedisClusterClient::ListClusters(
    google::cloud::redis::cluster::v1::ListClustersRequest request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->ListClusters(std::move(request));
}

StatusOr<google::cloud::redis::cluster::v1::Cluster>
CloudRedisClusterClient::GetCluster(std::string const& name, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::GetClusterRequest request;
  request.set_name(name);
  return connection_->GetCluster(request);
}

StatusOr<google::cloud::redis::cluster::v1::Cluster>
CloudRedisClusterClient::GetCluster(
    google::cloud::redis::cluster::v1::GetClusterRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->GetCluster(request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::UpdateCluster(
    google::cloud::redis::cluster::v1::Cluster const& cluster,
    google::protobuf::FieldMask const& update_mask, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::UpdateClusterRequest request;
  *request.mutable_cluster() = cluster;
  *request.mutable_update_mask() = update_mask;
  return connection_->UpdateCluster(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::UpdateCluster(
    NoAwaitTag, google::cloud::redis::cluster::v1::Cluster const& cluster,
    google::protobuf::FieldMask const& update_mask, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::UpdateClusterRequest request;
  *request.mutable_cluster() = cluster;
  *request.mutable_update_mask() = update_mask;
  return connection_->UpdateCluster(NoAwaitTag{}, request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::UpdateCluster(
    google::cloud::redis::cluster::v1::UpdateClusterRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->UpdateCluster(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::UpdateCluster(
    NoAwaitTag,
    google::cloud::redis::cluster::v1::UpdateClusterRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->UpdateCluster(NoAwaitTag{}, request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::UpdateCluster(
    google::longrunning::Operation const& operation, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->UpdateCluster(operation);
}

future<StatusOr<google::protobuf::Any>> CloudRedisClusterClient::DeleteCluster(
    std::string const& name, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::DeleteClusterRequest request;
  request.set_name(name);
  return connection_->DeleteCluster(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::DeleteCluster(
    NoAwaitTag, std::string const& name, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::DeleteClusterRequest request;
  request.set_name(name);
  return connection_->DeleteCluster(NoAwaitTag{}, request);
}

future<StatusOr<google::protobuf::Any>> CloudRedisClusterClient::DeleteCluster(
    google::cloud::redis::cluster::v1::DeleteClusterRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->DeleteCluster(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::DeleteCluster(
    NoAwaitTag,
    google::cloud::redis::cluster::v1::DeleteClusterRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->DeleteCluster(NoAwaitTag{}, request);
}

future<StatusOr<google::protobuf::Any>> CloudRedisClusterClient::DeleteCluster(
    google::longrunning::Operation const& operation, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->DeleteCluster(operation);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::CreateCluster(
    std::string const& parent,
    google::cloud::redis::cluster::v1::Cluster const& cluster,
    std::string const& cluster_id, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::CreateClusterRequest request;
  request.set_parent(parent);
  *request.mutable_cluster() = cluster;
  request.set_cluster_id(cluster_id);
  return connection_->CreateCluster(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::CreateCluster(
    NoAwaitTag, std::string const& parent,
    google::cloud::redis::cluster::v1::Cluster const& cluster,
    std::string const& cluster_id, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::CreateClusterRequest request;
  request.set_parent(parent);
  *request.mutable_cluster() = cluster;
  request.set_cluster_id(cluster_id);
  return connection_->CreateCluster(NoAwaitTag{}, request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::CreateCluster(
    google::cloud::redis::cluster::v1::CreateClusterRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->CreateCluster(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::CreateCluster(
    NoAwaitTag,
    google::cloud::redis::cluster::v1::CreateClusterRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->CreateCluster(NoAwaitTag{}, request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::CreateCluster(
    google::longrunning::Operation const& operation, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->CreateCluster(operation);
}

StatusOr<google::cloud::redis::cluster::v1::CertificateAuthority>
CloudRedisClusterClient::GetClusterCertificateAuthority(std::string const& name,
                                                        Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::GetClusterCertificateAuthorityRequest
      request;
  request.set_name(name);
  return connection_->GetClusterCertificateAuthority(request);
}

StatusOr<google::cloud::redis::cluster::v1::CertificateAuthority>
CloudRedisClusterClient::GetClusterCertificateAuthority(
    google::cloud::redis::cluster::v1::
        GetClusterCertificateAuthorityRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->GetClusterCertificateAuthority(request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::RescheduleClusterMaintenance(
    std::string const& name,
    google::cloud::redis::cluster::v1::RescheduleClusterMaintenanceRequest::
        RescheduleType reschedule_type,
    google::protobuf::Timestamp const& schedule_time, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::RescheduleClusterMaintenanceRequest
      request;
  request.set_name(name);
  request.set_reschedule_type(reschedule_type);
  *request.mutable_schedule_time() = schedule_time;
  return connection_->RescheduleClusterMaintenance(request);
}

StatusOr<google::longrunning::Operation>
CloudRedisClusterClient::RescheduleClusterMaintenance(
    NoAwaitTag, std::string const& name,
    google::cloud::redis::cluster::v1::RescheduleClusterMaintenanceRequest::
        RescheduleType reschedule_type,
    google::protobuf::Timestamp const& schedule_time, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::RescheduleClusterMaintenanceRequest
      request;
  request.set_name(name);
  request.set_reschedule_type(reschedule_type);
  *request.mutable_schedule_time() = schedule_time;
  return connection_->RescheduleClusterMaintenance(NoAwaitTag{}, request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::RescheduleClusterMaintenance(
    google::cloud::redis::cluster::v1::
        RescheduleClusterMaintenanceRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->RescheduleClusterMaintenance(request);
}

StatusOr<google::longrunning::Operation>
CloudRedisClusterClient::RescheduleClusterMaintenance(
    NoAwaitTag,
    google::cloud::redis::cluster::v1::
        RescheduleClusterMaintenanceRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->RescheduleClusterMaintenance(NoAwaitTag{}, request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::RescheduleClusterMaintenance(
    google::longrunning::Operation const& operation, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->RescheduleClusterMaintenance(operation);
}

StreamRange<google::cloud::redis::cluster::v1::BackupCollection>
CloudRedisClusterClient::ListBackupCollections(std::string const& parent,
                                               Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::ListBackupCollectionsRequest request;
  request.set_parent(parent);
  return connection_->ListBackupCollections(request);
}

StreamRange<google::cloud::redis::cluster::v1::BackupCollection>
CloudRedisClusterClient::ListBackupCollections(
    google::cloud::redis::cluster::v1::ListBackupCollectionsRequest request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->ListBackupCollections(std::move(request));
}

StatusOr<google::cloud::redis::cluster::v1::BackupCollection>
CloudRedisClusterClient::GetBackupCollection(std::string const& name,
                                             Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::GetBackupCollectionRequest request;
  request.set_name(name);
  return connection_->GetBackupCollection(request);
}

StatusOr<google::cloud::redis::cluster::v1::BackupCollection>
CloudRedisClusterClient::GetBackupCollection(
    google::cloud::redis::cluster::v1::GetBackupCollectionRequest const&
        request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->GetBackupCollection(request);
}

StreamRange<google::cloud::redis::cluster::v1::Backup>
CloudRedisClusterClient::ListBackups(std::string const& parent, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::ListBackupsRequest request;
  request.set_parent(parent);
  return connection_->ListBackups(request);
}

StreamRange<google::cloud::redis::cluster::v1::Backup>
CloudRedisClusterClient::ListBackups(
    google::cloud::redis::cluster::v1::ListBackupsRequest request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->ListBackups(std::move(request));
}

StatusOr<google::cloud::redis::cluster::v1::Backup>
CloudRedisClusterClient::GetBackup(std::string const& name, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::GetBackupRequest request;
  request.set_name(name);
  return connection_->GetBackup(request);
}

StatusOr<google::cloud::redis::cluster::v1::Backup>
CloudRedisClusterClient::GetBackup(
    google::cloud::redis::cluster::v1::GetBackupRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->GetBackup(request);
}

future<StatusOr<google::protobuf::Any>> CloudRedisClusterClient::DeleteBackup(
    std::string const& name, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::DeleteBackupRequest request;
  request.set_name(name);
  return connection_->DeleteBackup(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::DeleteBackup(
    NoAwaitTag, std::string const& name, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::DeleteBackupRequest request;
  request.set_name(name);
  return connection_->DeleteBackup(NoAwaitTag{}, request);
}

future<StatusOr<google::protobuf::Any>> CloudRedisClusterClient::DeleteBackup(
    google::cloud::redis::cluster::v1::DeleteBackupRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->DeleteBackup(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::DeleteBackup(
    NoAwaitTag,
    google::cloud::redis::cluster::v1::DeleteBackupRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->DeleteBackup(NoAwaitTag{}, request);
}

future<StatusOr<google::protobuf::Any>> CloudRedisClusterClient::DeleteBackup(
    google::longrunning::Operation const& operation, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->DeleteBackup(operation);
}

future<StatusOr<google::cloud::redis::cluster::v1::Backup>>
CloudRedisClusterClient::ExportBackup(
    google::cloud::redis::cluster::v1::ExportBackupRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->ExportBackup(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::ExportBackup(
    NoAwaitTag,
    google::cloud::redis::cluster::v1::ExportBackupRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->ExportBackup(NoAwaitTag{}, request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Backup>>
CloudRedisClusterClient::ExportBackup(
    google::longrunning::Operation const& operation, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->ExportBackup(operation);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::BackupCluster(std::string const& name, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::BackupClusterRequest request;
  request.set_name(name);
  return connection_->BackupCluster(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::BackupCluster(
    NoAwaitTag, std::string const& name, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::cloud::redis::cluster::v1::BackupClusterRequest request;
  request.set_name(name);
  return connection_->BackupCluster(NoAwaitTag{}, request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::BackupCluster(
    google::cloud::redis::cluster::v1::BackupClusterRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->BackupCluster(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::BackupCluster(
    NoAwaitTag,
    google::cloud::redis::cluster::v1::BackupClusterRequest const& request,
    Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->BackupCluster(NoAwaitTag{}, request);
}

future<StatusOr<google::cloud::redis::cluster::v1::Cluster>>
CloudRedisClusterClient::BackupCluster(
    google::longrunning::Operation const& operation, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->BackupCluster(operation);
}

StreamRange<google::cloud::location::Location>
CloudRedisClusterClient::ListLocations(
    google::cloud::location::ListLocationsRequest request, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->ListLocations(std::move(request));
}

StatusOr<google::cloud::location::Location>
CloudRedisClusterClient::GetLocation(
    google::cloud::location::GetLocationRequest const& request, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->GetLocation(request);
}

StreamRange<google::longrunning::Operation>
CloudRedisClusterClient::ListOperations(std::string const& name,
                                        std::string const& filter,
                                        Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::longrunning::ListOperationsRequest request;
  request.set_name(name);
  request.set_filter(filter);
  return connection_->ListOperations(request);
}

StreamRange<google::longrunning::Operation>
CloudRedisClusterClient::ListOperations(
    google::longrunning::ListOperationsRequest request, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->ListOperations(std::move(request));
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::GetOperation(
    std::string const& name, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::longrunning::GetOperationRequest request;
  request.set_name(name);
  return connection_->GetOperation(request);
}

StatusOr<google::longrunning::Operation> CloudRedisClusterClient::GetOperation(
    google::longrunning::GetOperationRequest const& request, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->GetOperation(request);
}

Status CloudRedisClusterClient::DeleteOperation(std::string const& name,
                                                Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::longrunning::DeleteOperationRequest request;
  request.set_name(name);
  return connection_->DeleteOperation(request);
}

Status CloudRedisClusterClient::DeleteOperation(
    google::longrunning::DeleteOperationRequest const& request, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->DeleteOperation(request);
}

Status CloudRedisClusterClient::CancelOperation(std::string const& name,
                                                Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  google::longrunning::CancelOperationRequest request;
  request.set_name(name);
  return connection_->CancelOperation(request);
}

Status CloudRedisClusterClient::CancelOperation(
    google::longrunning::CancelOperationRequest const& request, Options opts) {
  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), options_));
  return connection_->CancelOperation(request);
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace redis_cluster_v1
}  // namespace cloud
}  // namespace google
