// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/devtools/cloudtrace/v2/tracing.proto

#include "google/cloud/trace/v2/internal/trace_connection_impl.h"
#include "google/cloud/trace/v2/internal/trace_option_defaults.h"
#include "google/cloud/background_threads.h"
#include "google/cloud/common_options.h"
#include "google/cloud/grpc_options.h"
#include "google/cloud/internal/retry_loop.h"
#include <memory>
#include <utility>

namespace google {
namespace cloud {
namespace trace_v2_internal {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
namespace {

std::unique_ptr<trace_v2::TraceServiceRetryPolicy> retry_policy(
    Options const& options) {
  return options.get<trace_v2::TraceServiceRetryPolicyOption>()->clone();
}

std::unique_ptr<BackoffPolicy> backoff_policy(Options const& options) {
  return options.get<trace_v2::TraceServiceBackoffPolicyOption>()->clone();
}

std::unique_ptr<trace_v2::TraceServiceConnectionIdempotencyPolicy>
idempotency_policy(Options const& options) {
  return options.get<trace_v2::TraceServiceConnectionIdempotencyPolicyOption>()
      ->clone();
}

}  // namespace

TraceServiceConnectionImpl::TraceServiceConnectionImpl(
    std::unique_ptr<google::cloud::BackgroundThreads> background,
    std::shared_ptr<trace_v2_internal::TraceServiceStub> stub, Options options)
    : background_(std::move(background)),
      stub_(std::move(stub)),
      options_(internal::MergeOptions(std::move(options),
                                      TraceServiceConnection::options())) {}

Status TraceServiceConnectionImpl::BatchWriteSpans(
    google::devtools::cloudtrace::v2::BatchWriteSpansRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->BatchWriteSpans(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::devtools::cloudtrace::v2::BatchWriteSpansRequest const&
                 request) {
        return stub_->BatchWriteSpans(context, options, request);
      },
      *current, request, __func__);
}

StatusOr<google::devtools::cloudtrace::v2::Span>
TraceServiceConnectionImpl::CreateSpan(
    google::devtools::cloudtrace::v2::Span const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->CreateSpan(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::devtools::cloudtrace::v2::Span const& request) {
        return stub_->CreateSpan(context, options, request);
      },
      *current, request, __func__);
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace trace_v2_internal
}  // namespace cloud
}  // namespace google
